package appletdemo;

import com.adbs.querybuilder.*;
import com.adbs.syntax.*;
import javax.swing.*;
import java.awt.event.KeyEvent;
import java.awt.KeyboardFocusManager;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.HashSet;

public class QueryBuilderApplet extends javax.swing.JApplet
{
	private static final long serialVersionUID = 1L;

	private PlainTextSQLBuilder plainTextSQLBuilder1 = new PlainTextSQLBuilder();

	@Override
    public void init()
	{
        try
		{
            java.awt.EventQueue.invokeAndWait(new Runnable()
			{
                public void run()
				{
					// set native Look'n'Feel
					try
					{
						UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
					}
					catch (Exception ex)
					{
						System.out.println(ex.getMessage());
					}

                    initComponents();

					// set required syntax provider
					queryBuilder1.setSyntaxProvider(new MySQLSyntaxProvider());
					//set up SQL builder:
					plainTextSQLBuilder1.setQueryBuilder(queryBuilder1);

					// handle the notification from plainTextSQLBuilder1 about the query text has been changed
					plainTextSQLBuilder1.addSQLUpdatedEventListener(new SQLUpdatedEventListener()
					{
						@Override
						public void sqlUpdatedEventOccurred(SQLUpdatedEvent event)
						{
							try
							{
								jTextPane1.setText(plainTextSQLBuilder1.getSQL());
							}
							catch (QueryBuilderException ex)
							{
								JOptionPane.showMessageDialog(QueryBuilderApplet.this, ex.getMessage());
							}
						}
					});

					// Load metadata from XML file located in resources (for demonstration purposes)
					try
					{
						Class resourceClass = QueryBuilderApplet.class;
						InputStream stream = resourceClass.getResourceAsStream("sakila.xml");
						BufferedReader br = new BufferedReader(new InputStreamReader(stream));
						StringBuilder sb = new StringBuilder();
						String line = null;

						while ((line = br.readLine()) != null)
						{
							sb.append(line + "\n");
						}

						br.close();
						queryBuilder1.getMetadataContainer().loadFromXML(sb.toString());
					}
					catch (Exception ex)
					{
						Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
						JOptionPane.showMessageDialog(QueryBuilderApplet.this, ex.getMessage());
					}

					try
					{
						queryBuilder1.setSQL("Select film.film_id, film.title, film.description, actor.first_name, actor.last_name From film Inner Join film_actor On film_actor.film_id = film.film_id Inner Join actor On film_actor.actor_id = actor.actor_id");
					}
					catch (QueryBuilderException ex)
					{
						JOptionPane.showMessageDialog(QueryBuilderApplet.this, ex.getMessage());
					}


					// force the jTextPane1 to move the focus on the Tab key:

					KeyStroke tab = KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0);
					HashSet keystrokes = new HashSet();
					keystrokes.add(tab);
					jTextPane1.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, keystrokes);

					KeyStroke shiftTab = KeyStroke.getKeyStroke(KeyEvent.VK_TAB, KeyEvent.SHIFT_DOWN_MASK);
					keystrokes = new HashSet();
					keystrokes.add(shiftTab);
					jTextPane1.setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS, keystrokes);
                }
            });
        }
		catch (Exception ex)
		{
            ex.printStackTrace();
        }
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane1 = new javax.swing.JSplitPane();
        queryBuilder1 = new com.adbs.querybuilder.QueryBuilder();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        jTextPane2 = new javax.swing.JTextPane();

        setStub(null);

        jSplitPane1.setDividerLocation(450);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane1.setResizeWeight(1.0);
        jSplitPane1.setLeftComponent(queryBuilder1);

        jTextPane1.setFont(new java.awt.Font("Monospaced", 0, 12));
        jTextPane1.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextPane1FocusLost(evt);
            }
        });
        jScrollPane1.setViewportView(jTextPane1);

        jSplitPane1.setRightComponent(jScrollPane1);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        jPanel1.setPreferredSize(new java.awt.Dimension(821, 30));
        jPanel1.setLayout(new java.awt.BorderLayout());

        jScrollPane3.setBorder(null);
        jScrollPane3.setMinimumSize(new java.awt.Dimension(20, 20));
        jScrollPane3.setPreferredSize(new java.awt.Dimension(607, 20));

        jTextPane2.setBackground(java.awt.SystemColor.info);
        jTextPane2.setContentType("text/html");
        jTextPane2.setEditable(false);
        jTextPane2.setText("<html><b>Important information:</b> Before to run the applet in a browser you need to sign it. See 'How To Sign.txt' for instructions.</html>");
        jTextPane2.setMargin(new java.awt.Insets(7, 10, 3, 3));
        jScrollPane3.setViewportView(jTextPane2);

        jPanel1.add(jScrollPane3, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel1, java.awt.BorderLayout.PAGE_START);
    }// </editor-fold>//GEN-END:initComponents

	private void jTextPane1FocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_jTextPane1FocusLost
	{//GEN-HEADEREND:event_jTextPane1FocusLost
		// when the focus is going out of the text pane, update the query builder with new query text
		try
		{
			queryBuilder1.setSQL(jTextPane1.getText());
		}
		catch (QueryBuilderException ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
}//GEN-LAST:event_jTextPane1FocusLost

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTextPane jTextPane1;
    private javax.swing.JTextPane jTextPane2;
    private com.adbs.querybuilder.QueryBuilder queryBuilder1;
    // End of variables declaration//GEN-END:variables
}
