// This source code sample demonstrates imaplementation of custom expression builder,
// that can be called by additional "..." button from "Expression" and "Condition" column cells.
// Handle the CustomExpressionBuilderEvent to deal with this button.

package demo;

import com.adbs.querybuilder.*;
import com.adbs.syntax.*;
import java.awt.Font;
import java.awt.Frame;
import javax.swing.JOptionPane;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.EnumSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

public class MainFrame extends javax.swing.JFrame
{
	private static final long serialVersionUID = 1L;

	private PlainTextSQLBuilder plainTextSQLBuilder1 = new PlainTextSQLBuilder();

    public MainFrame()
	{
        initComponents();

		// center window on the desktop
		setLocationRelativeTo(null);
		// maximize window
		setExtendedState(Frame.MAXIMIZED_BOTH);

		// set required syntax provider
		queryBuilder1.setSyntaxProvider(new MySQLSyntaxProvider());
		//set up SQL builder:
		plainTextSQLBuilder1.setQueryBuilder(queryBuilder1);

		// handle the notification from plainTextSQLBuilder1 about the query text has been changed
		plainTextSQLBuilder1.addSQLUpdatedEventListener(new SQLUpdatedEventListener()
		{
			@Override
			public void sqlUpdatedEventOccurred(SQLUpdatedEvent event)
			{
				try
				{
					jTextPane1.setText(plainTextSQLBuilder1.getSQL());
				}
				catch (QueryBuilderException ex)
				{
					Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
				}
			}
		});

		// Load metadata from XML file located in resources (for demonstration purposes)
		try
		{
			Class resourceClass = MainFrame.class;
			InputStream stream = resourceClass.getResourceAsStream("sakila.xml");
			BufferedReader br = new BufferedReader(new InputStreamReader(stream));
			StringBuilder sb = new StringBuilder();
			String line = null;

			while ((line = br.readLine()) != null)
			{
				sb.append(line + "\n");
			}

			br.close();
			queryBuilder1.getMetadataContainer().loadFromXML(sb.toString());
		}
		catch (Exception ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}

		try
		{
			queryBuilder1.setSQL("Select film.film_id, film.title, film.description, actor.first_name, actor.last_name From film Inner Join film_actor On film_actor.film_id = film.film_id Inner Join actor On film_actor.actor_id = actor.actor_id where film.film_id > 0");
		}
		catch (QueryBuilderException ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
		
		
		// Set criteria list options:
		// ~~~~~~~~~~~~~~~~~~~~~~~~~~

		CriteriaListOptions clo = queryBuilder1.getCriteriaListOptions();

		clo.beginUpdate();

		// you can hide unwanted columns setting their width to 0.
		//clo.set***ColumnWidth(0);
		// you can limit the initial count of "Or..." columns
		//clo.setInitialOrColumnsCount(0);

		clo.setExpressionColumnWidth(200);

		// Enable "..." button for Expression and Conditions columns
		clo.setUseCustomExpressionBuilderFor(EnumSet.of(CustomExpressionBuilderColumns.Expression,
					CustomExpressionBuilderColumns.Conditions));

		// you also can disable direct editing in these columns
		//clo.setDisableDirectEditingIn(EnumSet.of(CustomExpressionBuilderColumns.Expression,
		//			CustomExpressionBuilderColumns.Conditions));

		clo.endUpdate();
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane1 = new javax.swing.JSplitPane();
        queryBuilder1 = new com.adbs.querybuilder.QueryBuilder();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Events Demo");

        jSplitPane1.setDividerLocation(350);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane1.setResizeWeight(1.0);

        queryBuilder1.addCustomExpressionBuilderEventListener(new com.adbs.querybuilder.CustomExpressionBuilderEventListener() {
            public void customExpressionBuilderEventOccurred(com.adbs.querybuilder.CustomExpressionBuilderEvent evt) {
                queryBuilder1CustomExpressionBuilderEventOccurred(evt);
            }
        });
        jSplitPane1.setLeftComponent(queryBuilder1);

        jTextPane1.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextPane1FocusLost(evt);
            }
        });
        jScrollPane1.setViewportView(jTextPane1);

        jSplitPane1.setRightComponent(jScrollPane1);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        jScrollPane2.setBorder(null);

        jTextArea1.setBackground(java.awt.SystemColor.info);
        jTextArea1.setColumns(20);
        jTextArea1.setEditable(false);
        jTextArea1.setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N
        jTextArea1.setForeground(new java.awt.Color(128, 0, 0));
        jTextArea1.setRows(1);
        jTextArea1.setText("To see the custom expression builder click on \"...\" button in cells of Expression and Condition columns.");
        jTextArea1.setAutoscrolls(false);
        jTextArea1.setFocusable(false);
        jTextArea1.setMargin(new java.awt.Insets(5, 20, 5, 5));
        jScrollPane2.setViewportView(jTextArea1);

        getContentPane().add(jScrollPane2, java.awt.BorderLayout.PAGE_START);

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void jTextPane1FocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_jTextPane1FocusLost
	{//GEN-HEADEREND:event_jTextPane1FocusLost
		// when the focus is going out of the text pane, update the query builder with new query text
		try
		{
			queryBuilder1.setSQL(jTextPane1.getText());
		}
		catch (QueryBuilderException ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
}//GEN-LAST:event_jTextPane1FocusLost

	private void queryBuilder1CustomExpressionBuilderEventOccurred(com.adbs.querybuilder.CustomExpressionBuilderEvent evt)//GEN-FIRST:event_queryBuilder1CustomExpressionBuilderEventOccurred
	{//GEN-HEADEREND:event_queryBuilder1CustomExpressionBuilderEventOccurred
		// Show simple text editor using JOptionPane.showOptionDialog().
		// You can use any custom dialog instead.

		JTextArea textArea = new JTextArea(10, 50);

		textArea.setText(evt.value);
		textArea.selectAll();
		textArea.setFont(new Font("SansSerif", Font.PLAIN, 11));

		int result = JOptionPane.showOptionDialog(this, new JScrollPane(textArea), "Custom Editor",
				JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null);

		if (result == JOptionPane.OK_OPTION)
		{
			// send changed text to the query builder
			evt.value = textArea.getText();
		}
	}//GEN-LAST:event_queryBuilder1CustomExpressionBuilderEventOccurred

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextPane jTextPane1;
    private com.adbs.querybuilder.QueryBuilder queryBuilder1;
    // End of variables declaration//GEN-END:variables

}
