/************************************************************************
 * This sample demonstrates the implementation of custom metadata tree. *
 ************************************************************************/ 

package custommetadatatree;

import com.adbs.querybuilder.*;
import com.adbs.syntax.*;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.TransferHandler;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

public class MainFrame extends javax.swing.JFrame
{
	public MainFrame()
	{
        initComponents();

		// Center window on the desktop
		setLocationRelativeTo(null);

		// Hide QueryBuilder's trees
		queryBuilder1.getMetadataTreeOptions().setTreeVisible(false);
		queryBuilder1.getQueryStructureTreeOptions().setTreeVisible(false);
				
		// Set appropriate syntax provider
		queryBuilder1.setSyntaxProvider(new MySQLSyntaxProvider());
				
		// Load metadata from XML snapshot located in resources (for demonstration purposes)
		try
		{
			InputStream stream = MainFrame.class.getResourceAsStream("sakila.xml");
			BufferedReader br = new BufferedReader(new InputStreamReader(stream));
			StringBuilder sb = new StringBuilder();
			String line;
			
			while ((line = br.readLine()) != null)
				sb.append(line).append("\n");
			
			br.close();
			
			queryBuilder1.setOfflineMode(true);
			queryBuilder1.getMetadataContainer().loadFromXML(sb.toString());
		}
		catch (Exception ex)
		{
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
		
		// Fill the tree with some objects from loaded metadata
		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("root");
		rootNode.add(new DefaultMutableTreeNode("sakila.film"));
		rootNode.add(new DefaultMutableTreeNode("sakila.film_category"));
		rootNode.add(new DefaultMutableTreeNode("sakila.film_actor"));
		
		// Setup tree
		metadataTree.setModel(new DefaultTreeModel(rootNode));
		metadataTree.setRootVisible(false);
		metadataTree.setDragEnabled(true);
				
		// Handle tree mouse clicks
		metadataTree.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseClicked(MouseEvent e)
			{
				// Add selected object on double click
				if (e.getClickCount() == 2)
				{
					TreePath selectedPath = metadataTree.getSelectionPath();

					if (selectedPath != null)
					{
						DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
						String objectName = (String) selectedNode.getUserObject();

						UnionSubQuery unionSubQuery = queryBuilder1.getActiveSubquery().getActiveUnionSubquery();
						unionSubQuery.addObjectWithFK(objectName, "");
					}
				}
			}
		});
		
		// Handle tree items drag
		metadataTree.setTransferHandler(new TransferHandler()
		{
			@Override
			public boolean canImport(TransferSupport support)
			{
				return false;
			}

			@Override
			public int getSourceActions(JComponent c)
			{
				return TransferHandler.LINK;
			}

			@Override
			protected Transferable createTransferable(JComponent c)
			{
				// We should return Transferable object containing list of fully qualified object names 
				// separated by '\n' char:
				
                TreePath[] selectedPaths = metadataTree.getSelectionPaths();

				if (selectedPaths.length > 0)
				{
					StringBuilder stringBuilder = new StringBuilder();

					for (TreePath path: selectedPaths)
					{
						String objectName = (String) ((DefaultMutableTreeNode) path.getLastPathComponent()).getUserObject();
						
						stringBuilder.append(objectName);
						stringBuilder.append("\n");
					}

					return new StringSelection(stringBuilder.toString());
				}
				else
				{
					return null;
				}
			}
		});
    }
	
	
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {

        jSplitPane1 = new javax.swing.JSplitPane();
        queryBuilder1 = new com.adbs.querybuilder.QueryBuilder();
        jScrollPane1 = new javax.swing.JScrollPane();
        metadataTree = new JTree();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Events Demo");

        jSplitPane1.setDividerLocation(250);
        jSplitPane1.setResizeWeight(1.0);
        jSplitPane1.setRightComponent(queryBuilder1);

        jScrollPane1.setViewportView(metadataTree);

        jSplitPane1.setLeftComponent(jScrollPane1);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTree metadataTree;
    private com.adbs.querybuilder.QueryBuilder queryBuilder1;
    // End of variables declaration//GEN-END:variables

}
