package eventsdemo;

import com.adbs.querybuilder.*;
import com.adbs.syntax.*;
import javax.swing.JOptionPane;
import javax.swing.JMenuItem;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.logging.Level;
import java.util.logging.Logger;

public class MainFrame extends javax.swing.JFrame
{
	private static final long serialVersionUID = 1L;

	private PlainTextSQLBuilder plainTextSQLBuilder1 = new PlainTextSQLBuilder();

    public MainFrame()
	{
        initComponents();

		// center window on the desktop
		setLocationRelativeTo(null);

		// set required syntax provider
		queryBuilder1.setSyntaxProvider(new MySQLSyntaxProvider());
		//set up SQL builder:
		plainTextSQLBuilder1.setQueryBuilder(queryBuilder1);

		// handle the notification from plainTextSQLBuilder1 about the query text has been changed
		plainTextSQLBuilder1.addSQLUpdatedEventListener(new SQLUpdatedEventListener()
		{
			@Override
			public void sqlUpdatedEventOccurred(SQLUpdatedEvent event)
			{
				try
				{
					jTextPane1.setText(plainTextSQLBuilder1.getSQL());
				}
				catch (QueryBuilderException ex)
				{
					Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
				}
			}
		});

		// Load metadata from XML file located in resources (for demonstration purposes)
		try
		{
			Class resourceClass = MainFrame.class;
			InputStream stream = resourceClass.getResourceAsStream("sakila.xml");
			BufferedReader br = new BufferedReader(new InputStreamReader(stream));
			StringBuilder sb = new StringBuilder();
			String line = null;

			while ((line = br.readLine()) != null)
			{
				sb.append(line + "\n");
			}

			br.close();
			queryBuilder1.getMetadataContainer().loadFromXML(sb.toString());
		}
		catch (Exception ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}

		try
		{
			queryBuilder1.setSQL("Select film.film_id, film.title, film.description, actor.first_name, actor.last_name From film Inner Join film_actor On film_actor.film_id = film.film_id Inner Join actor On film_actor.actor_id = actor.actor_id");
		}
		catch (QueryBuilderException ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane1 = new javax.swing.JSplitPane();
        queryBuilder1 = new com.adbs.querybuilder.QueryBuilder();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Events Demo");

        jSplitPane1.setDividerLocation(350);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane1.setResizeWeight(1.0);

        queryBuilder1.addBeforePropertiesDialogEventListener(new com.adbs.querybuilder.BeforePropertiesDialogEventListener() {
            public void beforePropertiesDialogEventOccurred(com.adbs.querybuilder.BeforePropertiesDialogEvent evt) {
                queryBuilder1BeforePropertiesDialogEventOccurred(evt);
            }
        });
        queryBuilder1.addLinkDeletingEventListener(new com.adbs.querybuilder.LinkDeletingEventListener() {
            public void linkDeletingEventOccurred(com.adbs.querybuilder.LinkDeletingEvent evt) {
                queryBuilder1LinkDeletingEventOccurred(evt);
            }
        });
        queryBuilder1.addDiagramObjectAddingEventListener(new com.adbs.querybuilder.DiagramObjectAddingEventListener() {
            public void diagramObjectAddingEventOccurred(com.adbs.querybuilder.DiagramObjectAddingEvent evt) {
                queryBuilder1DiagramObjectAddingEventOccurred(evt);
            }
        });
        queryBuilder1.addGridCellValueChangingEventListener(new com.adbs.querybuilder.GridCellValueChangingEventListener() {
            public void gridCellValueChangingEventOccurred(com.adbs.querybuilder.GridCellValueChangingEvent evt) {
                queryBuilder1GridCellValueChangingEventOccurred(evt);
            }
        });
        queryBuilder1.addLinkCreatingEventListener(new com.adbs.querybuilder.LinkCreatingEventListener() {
            public void linkCreatingEventOccurred(com.adbs.querybuilder.LinkCreatingEvent evt) {
                queryBuilder1LinkCreatingEventOccurred(evt);
            }
        });
        queryBuilder1.addValidatePopupMenuEventListener(new com.adbs.querybuilder.ValidatePopupMenuEventListener() {
            public void validatePopupMenuEventOccurred(com.adbs.querybuilder.ValidatePopupMenuEvent evt) {
                queryBuilder1ValidatePopupMenuEventOccurred(evt);
            }
        });
        queryBuilder1.addDatasourceFieldAddingEventListener(new com.adbs.querybuilder.DatasourceFieldAddingEventListener() {
            public void datasourceFieldAddingEventOccurred(com.adbs.querybuilder.DatasourceFieldAddingEvent evt) {
                queryBuilder1DatasourceFieldAddingEventOccurred(evt);
            }
        });
        jSplitPane1.setLeftComponent(queryBuilder1);

        jTextPane1.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextPane1FocusLost(evt);
            }
        });
        jScrollPane1.setViewportView(jTextPane1);

        jSplitPane1.setRightComponent(jScrollPane1);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        jScrollPane2.setBorder(null);

        jTextArea1.setBackground(java.awt.SystemColor.info);
        jTextArea1.setColumns(20);
        jTextArea1.setEditable(false);
        jTextArea1.setFont(new java.awt.Font("Tahoma", 1, 11));
        jTextArea1.setForeground(new java.awt.Color(128, 0, 0));
        jTextArea1.setRows(9);
        jTextArea1.setText("This applicaton demonstrates following QueryBuilder events:\n- DiagramObjectAdding (try to add any object on the diagram pane);\n- DatasourceFieldAdding (try to check a field)\n- GrigCellValueChanging (try to edit a cell in the Alias column);\n- LinkCreating (try to create a link between any fields of two different datasource objects);\n- LinkDeleting (try to delete a link);\n- BeforePropertiesDialog (try to edit properties of query, datasource or link).\n- ValidateContextMenu (try to open any context menu).\nSee the source code for detailed implementations.");
        jTextArea1.setAutoscrolls(false);
        jTextArea1.setFocusable(false);
        jTextArea1.setMargin(new java.awt.Insets(2, 5, 2, 2));
        jScrollPane2.setViewportView(jTextArea1);

        getContentPane().add(jScrollPane2, java.awt.BorderLayout.PAGE_START);

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void jTextPane1FocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_jTextPane1FocusLost
	{//GEN-HEADEREND:event_jTextPane1FocusLost
		// when the focus is going out of the text pane, update the query builder with new query text
		try
		{
			queryBuilder1.setSQL(jTextPane1.getText());
		}
		catch (QueryBuilderException ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
}//GEN-LAST:event_jTextPane1FocusLost

	private void queryBuilder1DiagramObjectAddingEventOccurred(com.adbs.querybuilder.DiagramObjectAddingEvent evt)//GEN-FIRST:event_queryBuilder1DiagramObjectAddingEventOccurred
	{//GEN-HEADEREND:event_queryBuilder1DiagramObjectAddingEventOccurred
		//
		// DiagramObjectAdding event allows to prevent adding of objects on the diagram pane.
		//
		String msg = "Do you want to add the object " + evt.fromObject.getMetadataObject().getNameStr() +
				" on the diagram pane?";

		if (JOptionPane.showConfirmDialog(this, msg, "DiagramObjectAdding event handler",
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) != JOptionPane.YES_OPTION)
		{
			evt.abort = true;
		}
	}//GEN-LAST:event_queryBuilder1DiagramObjectAddingEventOccurred

	private void queryBuilder1DatasourceFieldAddingEventOccurred(com.adbs.querybuilder.DatasourceFieldAddingEvent evt)//GEN-FIRST:event_queryBuilder1DatasourceFieldAddingEventOccurred
	{//GEN-HEADEREND:event_queryBuilder1DatasourceFieldAddingEventOccurred
		//
		// DatasourceFieldAdding event allows to prevent selecting of a field.
		//
		String msg = "Do you want to add the field \"" + evt.field.getNameStr() + "\" to the query?";

		if (JOptionPane.showConfirmDialog(this, msg, "DatasourceFieldAdding event handler",
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) != JOptionPane.YES_OPTION)
		{
			evt.abort = true;
		}
	}//GEN-LAST:event_queryBuilder1DatasourceFieldAddingEventOccurred

	private void queryBuilder1GridCellValueChangingEventOccurred(com.adbs.querybuilder.GridCellValueChangingEvent evt)//GEN-FIRST:event_queryBuilder1GridCellValueChangingEventOccurred
	{//GEN-HEADEREND:event_queryBuilder1GridCellValueChangingEventOccurred
		// GridCellValueChanging event allows to prevent the cell value changing or modify the new cell value.
		// Note: Some columns hide/unhide dynamically but this does not affect the column index in the event parameters -
		//       it includes hidden columns.
		if (evt.column == 1) // Prevent changes in the Expression column.
		{
			evt.abort = true;
		}
		else if (evt.column == 3) // Alias column. Lets add the underscore char in the beginning of all aliases, for example.
		{
			if (evt.newValue != null && evt.newValue instanceof String)
			{
				String s = (String) evt.newValue;

				if (s.length() > 0 && !s.startsWith("_"))
				{
					s = "_" + s;
					evt.newValue = s;
				}
			}
		}
	}//GEN-LAST:event_queryBuilder1GridCellValueChangingEventOccurred

	private void queryBuilder1LinkCreatingEventOccurred(com.adbs.querybuilder.LinkCreatingEvent evt)//GEN-FIRST:event_queryBuilder1LinkCreatingEventOccurred
	{//GEN-HEADEREND:event_queryBuilder1LinkCreatingEventOccurred
		//
		// LinkCreating event allows to prevent link creation
		//
		String msg = String.format("Do you want to create the link between %1$s.%2$s and %3$s.%4$s?",
				evt.fromDatasource.getMetadataObject().getNameStr(), evt.fromField.getNameStr(),
				evt.toDatasource.getMetadataObject().getNameStr(), evt.toField.getNameStr());

		if (JOptionPane.showConfirmDialog(this, msg, "LinkCreating event handler",
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) != JOptionPane.YES_OPTION)
		{
			evt.abort = true;
		}
	}//GEN-LAST:event_queryBuilder1LinkCreatingEventOccurred

	private void queryBuilder1LinkDeletingEventOccurred(com.adbs.querybuilder.LinkDeletingEvent evt)//GEN-FIRST:event_queryBuilder1LinkDeletingEventOccurred
	{//GEN-HEADEREND:event_queryBuilder1LinkDeletingEventOccurred
		//
		// LinkDeleting event allows to prevent link deletion.
		//
		String msg = String.format("Do you want to delete the link between %1$s and %2$s?",
				evt.link.getLeftField().getQualifiedName(), evt.link.getRightField().getQualifiedName());

		if (JOptionPane.showConfirmDialog(this, msg, "LinkDeleting event handler",
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) != JOptionPane.YES_OPTION)
		{
			evt.abort = true;
		}
	}//GEN-LAST:event_queryBuilder1LinkDeletingEventOccurred

	private void queryBuilder1BeforePropertiesDialogEventOccurred(com.adbs.querybuilder.BeforePropertiesDialogEvent evt)//GEN-FIRST:event_queryBuilder1BeforePropertiesDialogEventOccurred
	{//GEN-HEADEREND:event_queryBuilder1BeforePropertiesDialogEventOccurred
		//
		// BeforePropertiesDialog event allows to prevent various properties dialogs from showing.
		//
		String msg = "";

		if (evt.owner instanceof UnionSubQuery) // Query Properties dialog (invoked from the diagram pane context menu)
		{
			msg = "Do you want to show Query Properties dialog?";
		}
		else if (evt.owner instanceof Link) // Link Properties dialog (invoked from the link context menu)
		{
			msg = "Do you want to show Link Properties dialog?";
		}
		else if (evt.owner instanceof DataSourceObject) // Datasource Properties dialog (invoked from the datasource context menu)
		{
			msg = "Do you want to show Datasource Properties dialog?";
		}

		if (!msg.equals(""))
		{
			if (JOptionPane.showConfirmDialog(this, msg, "BeforePropertiesDialog event handler",
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) != JOptionPane.YES_OPTION)
			{
				evt.abort = true;
			}
		}
	}//GEN-LAST:event_queryBuilder1BeforePropertiesDialogEventOccurred

	private void queryBuilder1ValidatePopupMenuEventOccurred(com.adbs.querybuilder.ValidatePopupMenuEvent evt)//GEN-FIRST:event_queryBuilder1ValidatePopupMenuEventOccurred
	{//GEN-HEADEREND:event_queryBuilder1ValidatePopupMenuEventOccurred
		//
		// ValidatePopupMenu event allows to modify or hide any query builder popup menu.
		//

		// Add custom menu item to every context menu
		evt.menu.addSeparator();
		evt.menu.add(new JMenuItem("Custom Item 1"));

		if (evt.owner instanceof Link) // Link context menu
		{
			// Add another item in the Link's menu
			evt.menu.add(new JMenuItem("Custom Item 2"));
		}
		else if (evt.owner instanceof DataSourceObject) // Datasource context menu
		{

		}
		else if (evt.owner instanceof UnionSubQuery)
		{
			if (evt.component instanceof com.adbs.querybuilder.DiagramPane) // diagram pane context menu
			{

			}
			else if (evt.component instanceof com.adbs.querybuilder.UnionButton) // context menu of the union button ("Q" button on the navigation bar)
			{

			}
			else if (evt.component instanceof com.adbs.querybuilder.UnionOperator) // context menu of the union operator (union type symbol on the navigation bar)
			{
				// Delete all items from this menu.
				// The menu will not be shown.
				evt.menu.removeAll();
			}
		}
		else if (evt.owner instanceof UnionGroup)
		{
			if (evt.component instanceof LeftBracket) // context menu of the left bracket ("(" symbol on the navigation bar)
			{

			}
			else if (evt.component instanceof RightBracket) // context menu of the right bracket (")" symbol on the navigation bar)
			{

			}
		}
		else if (evt.owner instanceof CriteriaItem) // context menu of the row of the criteria list (grid)
		{

		}
	}//GEN-LAST:event_queryBuilder1ValidatePopupMenuEventOccurred

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextPane jTextPane1;
    private com.adbs.querybuilder.QueryBuilder queryBuilder1;
    // End of variables declaration//GEN-END:variables

}
