package generaldemo.connection_dialogs;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.sql.Connection;
import java.sql.DriverManager;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.Preferences;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileNameExtensionFilter;

public class SQLiteConnectionDialog extends javax.swing.JDialog
{
	private static final long serialVersionUID = 1L;

	private boolean dialogResult = false;

	public String url;

	public SQLiteConnectionDialog()
	{
        super((java.awt.Frame) null, true);
        
		initComponents();

		setLocationRelativeTo(null);

		Preferences prefs = Preferences.userNodeForPackage(SQLiteConnectionDialog.class);

		tfDatabase.setText(prefs.get("sqlite:database", ""));

		getRootPane().setDefaultButton(bConnect);

		AbstractAction escape = new AbstractAction()
		{
			@Override
			public void actionPerformed(ActionEvent actionEvent)
			{
				setVisible(false);
			}
		};

		// make the dialog to close on the Escape key press
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "ESCAPE");
		rootPane.getActionMap().put("ESCAPE", escape);
    }

	public boolean showDialog()
	{
		setVisible(true);

		return dialogResult;
	}

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        tfDatabase = new javax.swing.JTextField();
        bCancel = new javax.swing.JButton();
        bConnect = new javax.swing.JButton();
        bBrowse = new javax.swing.JButton();

        setTitle("Connect to SQLite database");
        setModal(true);

        jLabel1.setText("Database");

        bCancel.setText("Cancel");
        bCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bCancelActionPerformed(evt);
            }
        });

        bConnect.setText("Connect");
        bConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bConnectActionPerformed(evt);
            }
        });

        bBrowse.setText("...");
        bBrowse.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bBrowseActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(bConnect, javax.swing.GroupLayout.PREFERRED_SIZE, 81, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(bCancel, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(jLabel1)
                        .addGap(7, 7, 7)
                        .addComponent(tfDatabase, javax.swing.GroupLayout.DEFAULT_SIZE, 204, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(bBrowse, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {bCancel, bConnect});

        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(tfDatabase, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1)
                    .addComponent(bBrowse))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 42, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(bConnect)
                    .addComponent(bCancel))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void bConnectActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_bConnectActionPerformed
	{//GEN-HEADEREND:event_bConnectActionPerformed
		url = "jdbc:sqlite:/" + tfDatabase.getText();

		// try to connect
		try
		{
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

			Class.forName("org.sqlite.JDBC");
			Connection connection = DriverManager.getConnection(url);
			connection.close();

			Preferences prefs = Preferences.userNodeForPackage(SQLiteConnectionDialog.class);
			prefs.put("sqlite:database", tfDatabase.getText());

			dialogResult = true;
			setVisible(false);
		}
		catch (ClassNotFoundException ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, "Failed to load driver: \n" + ex.getMessage());
		}
		catch (Exception ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
		finally
		{
			setCursor(Cursor.getDefaultCursor());
		}
	}//GEN-LAST:event_bConnectActionPerformed

	private void bCancelActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_bCancelActionPerformed
	{//GEN-HEADEREND:event_bCancelActionPerformed
		Preferences prefs = Preferences.userNodeForPackage(SQLiteConnectionDialog.class);
		prefs.put("sqlite:database", tfDatabase.getText());

		dialogResult = false;
		setVisible(false);
	}//GEN-LAST:event_bCancelActionPerformed

	private void bBrowseActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_bBrowseActionPerformed
	{//GEN-HEADEREND:event_bBrowseActionPerformed
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
		fileChooser.setDialogTitle("Select SQLite database file");
		fileChooser.setAcceptAllFileFilterUsed(true);
		fileChooser.setFileFilter(new FileNameExtensionFilter("SL3 files", "sl3"));

		if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
		{
			tfDatabase.setText(fileChooser.getSelectedFile().getPath());
		}
}//GEN-LAST:event_bBrowseActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bBrowse;
    private javax.swing.JButton bCancel;
    private javax.swing.JButton bConnect;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JTextField tfDatabase;
    // End of variables declaration//GEN-END:variables
}
